/**
*****************************************************************************
**
**  File        : main.c
**
**  Abstract    : main function.
**
**  Functions   : main
**
**
**
**
*****************************************************************************
*/

/* Includes */
#include <stddef.h>
#include "stm32f10x.h"
#include "lib_pcd8544.h"
#include "macro.h"

//#define CopyBit(var1, index1, var2, index2) var1 = ((var2 & (1 << index2))?(var1 | (0x01 << index1)):(var1 & ~(0x01 << index1)))
//#define CopyBit(var1, index1, var2, index2) if((var2 & (1 << index2)) != 0){var1 = 1;}else{var1 = 0;}//~(0x01 << index1);}

/* Private typedef */
/* Private define  */
/* Private macro */
/* Private variables */
uint32_t freq = 0;		//zmienna przechowuje czestotliwosc taktowania rdzenia mikrokontrolera
uint8_t set_pos = 0;	//zmienna przechowuje poloenie wskanika aktualnie modyfikowanej wartoci
uint8_t req_alarm = 0;	//zmienna informujca o wywoaniu alarmu
uint32_t rtc_time = 0;	//zmienna przechowuje aktualny zcas
uint32_t rtc_alarm = 0;	//zmienna przechowuje czas alarmu
uint8_t alarm_state = 0;//zmienna informujca o wczeniu/wyczeniu obsugi alarmu

/* Private function prototypes */
/* Private functions */
void Configuration_RCC(void);
void Configuration_RTC(void);
void Configuration_Interrupts(void);
void Configuration_GPIO(void);

/**
**===========================================================================
**
**  Abstract: main program
**
**===========================================================================
*/
int main(void)
{
	//przeprowadzenie inicjalizacji
	Configuration_RCC();
	Configuration_RTC();
	Configuration_Interrupts();
	Configuration_GPIO();


	//inicjalizacja i obsuga wywietlacza
	PCD8544_INIT();
	PCD8544_LCDClear();
	PCD8544_LCDWriteText("ZEGAREK", 21, 0);
	PCD8544_LCDWriteText("Czas  00:00:00", 0, 1);
	PCD8544_LCDWriteText("Alarm 00:00:00", 0, 2);
	PCD8544_LCDWriteText("AlarmState  ON", 0, 3);
	PCD8544_LCDWrite();

	while(1)
	{
		//wywietlanie informacji na wywietlaczu w nieskoczonej ptli

		//wypisanie aktualnego czasu
		PCD8544_LCDWriteSign(rtc_time % 86400 / 36000 + 0x30, 36, 1);
		PCD8544_LCDWriteSign(rtc_time % 36000 / 3600 + 0x30, 42, 1);
		PCD8544_LCDWriteSign(':', 48, 1);
		PCD8544_LCDWriteSign(rtc_time % 3600 / 600 + 0x30, 54, 1);
		PCD8544_LCDWriteSign(rtc_time % 600 / 60 + 0x30, 60, 1);
		PCD8544_LCDWriteSign(':', 66, 1);
		PCD8544_LCDWriteSign(rtc_time % 60 / 10 + 0x30, 72, 1);
		PCD8544_LCDWriteSign(rtc_time % 10 + 0x30, 78, 1);

		//wypisanie aktualnego czasu alarmu
		PCD8544_LCDWriteSign(rtc_alarm / 36000 + 0x30, 36, 2);
		PCD8544_LCDWriteSign(rtc_alarm % 36000 / 3600 + 0x30, 42, 2);
		PCD8544_LCDWriteSign(':', 48, 2);
		PCD8544_LCDWriteSign(rtc_alarm % 3600 / 600 + 0x30, 54, 2);
		PCD8544_LCDWriteSign(rtc_alarm % 600 / 60 + 0x30, 60, 2);
		PCD8544_LCDWriteSign(':', 66, 2);
		PCD8544_LCDWriteSign(rtc_alarm % 60 / 10 + 0x30, 72, 2);
		PCD8544_LCDWriteSign(rtc_alarm % 10 + 0x30, 78, 2);

		//wypisanie stanu on/off alarmu
		if(alarm_state == 0)
		{
			PCD8544_LCDWriteSign('O', 66, 3);
			PCD8544_LCDWriteSign('F', 72, 3);
			PCD8544_LCDWriteSign('F', 78, 3);
		}
		else
		{
			PCD8544_LCDWriteSign(' ', 66, 3);
			PCD8544_LCDWriteSign('O', 72, 3);
			PCD8544_LCDWriteSign('N', 78, 3);
		}

		//podwietlenie edytowanego elementu
		switch(set_pos)
		{
			//czas
			case 1 : PCD8544_LCDInvert(36,1,41,1); break;
			case 2 : PCD8544_LCDInvert(42,1,47,1); break;
			case 3 : PCD8544_LCDInvert(54,1,59,1); break;
			case 4 : PCD8544_LCDInvert(60,1,65,1); break;
			case 5 : PCD8544_LCDInvert(72,1,77,1); break;
			case 6 : PCD8544_LCDInvert(78,1,83,1); break;
			//czas alarmu
			case 7 : PCD8544_LCDInvert(36,2,41,2); break;
			case 8 : PCD8544_LCDInvert(42,2,47,2); break;
			case 9 : PCD8544_LCDInvert(54,2,59,2); break;
			case 10 : PCD8544_LCDInvert(60,2,65,2); break;
			case 11 : PCD8544_LCDInvert(72,2,77,2); break;
			case 12 : PCD8544_LCDInvert(78,2,83,2); break;
			//on/off alarm
			case 13 :
				if(alarm_state == 1)
					{PCD8544_LCDInvert(72,3,83,3);}
				else
					{PCD8544_LCDInvert(66,3,83,3);}
			break;

			default : break;
		}
		PCD8544_LCDWrite();


	}
}

void Configuration_RCC(void)
{
	ErrorStatus HSEStartUpStatus;

	//Reset ustawien RCC
	RCC_DeInit();

	//Wlaczaenie HSE
	RCC_HSEConfig(RCC_HSE_ON);

	//Zaczekaj az HSE zostaeni zainicjowany
	HSEStartUpStatus = RCC_WaitForHSEStartUp();

	//jesli HSE zosta zainicjalizowany poprawnie
	if(HSEStartUpStatus == SUCCESS)
	{
		//Opoznienie dla pamieci Flash
		FLASH_SetLatency(FLASH_Latency_2);

		//Wlaczenie buffora pamieci flash
		FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

		//konfiguracja preskalera AHB
		//HCLK = SYSCLK
		RCC_HCLKConfig(RCC_SYSCLK_Div1);

		//konfigracja preskalera APB2
	    //PCLK2 = HCLK = 72MHz, czestotliwosz 72MHz bedzie uzyskana dopiero po pelnej inicjalizacji RCC
		RCC_PCLK2Config(RCC_HCLK_Div1);

	    //konfigracja preskalera APB1
		//PCLK1 = HCLK/2 = 36MHz, czestotliwosz 36MHz bedzie uzyskana dopiero po pelnej inicjalizacji RCC
		RCC_PCLK1Config(RCC_HCLK_Div2);

	    //konfiguracja PLLLCK
		//PLLCLK = HSE / 1 * 9 = 72 MHz
		RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

		//Enable PLL
	    RCC_PLLCmd(ENABLE);

	    //odczekanie az PLL bedzie gotowy do pracy
	    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET){}

	    //konfiguracja SW
		//wybranie PLL jako SYSCLK
		RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

	    //odczekanie az PLLCLK bedzie sygnaem zegarowym mikrokontrolera
	    while((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != (uint32_t)0x08){}
	}

	//wczenie zegarw wykorzystywanych peryferii APB1
	RCC_APB1PeriphClockCmd(
		RCC_APB1Periph_BKP | //wlaczenie zegara dla funkcji podtrzymania pamieci
		RCC_APB1Periph_PWR   //wlaczenie zegara dla funkcji power
		, ENABLE);

	//wczenie zegarw wykorzystywanych peryferii APB2
	RCC_APB2PeriphClockCmd(
		RCC_APB2Periph_GPIOA |
		RCC_APB2Periph_GPIOB |
		RCC_APB2Periph_GPIOC |
		RCC_APB2Periph_AFIO
		, ENABLE);
}

void Configuration_RTC(void)
{
	//wczenie dostpu do BKP
	PWR_BackupAccessCmd(ENABLE);

	//deinicjalizacja BPK
	BKP_DeInit();

	//wejcie do trybu edycji RTC
	RTC_EnterConfigMode();

	//wczenie LSE
	RCC_LSEConfig(RCC_LSE_ON);

	//odczekanie na poprawne wczenie LSE
	while(RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET){};

	//RTCCLK = LSE = 32768 Hz
	RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);

	//wczenie zegara RTC
	RCC_RTCCLKCmd(ENABLE);

	//dczekanie na synchronizacje
	RTC_WaitForSynchro();

	//odczekanie na zakoczenie operacji
	RTC_WaitForLastTask();

	//ustawienie preskalera na 32768 taktw
	RTC_SetPrescaler(32767);

	//odczekanie na zakoczenie operacji
	RTC_WaitForLastTask();

	//wyjscie z trybu edycji
	RTC_ExitConfigMode();

	//ustawienie licznika RTC na 12h00m00s
	RTC_SetCounter(43200);

	//zresetowanie flagi SECF rejestru RTC->CRL, wymagane do poprawnego zapisu rejestru alarmu
	RTC_ClearFlag(RTC_FLAG_SEC);
	while(RTC_GetFlagStatus(RTC_FLAG_SEC) == RESET){};

	//odczekanie na zakoczenie operacji na RTC
	RTC_WaitForLastTask();

	//ustawienie rejestru alarmu RTC na 12h00m00s
	RTC_SetAlarm(43200);
	//odczekanie na zakoczenie operacji na RTC
	RTC_WaitForLastTask();
}

void Configuration_Interrupts(void)
{
	//zdefiniowanie struktury wykorzystywanej do konfiguracji przerwani
	NVIC_InitTypeDef NVIC_InitStructure;
	EXTI_InitTypeDef EXTI_InitStructure;
	RCC_ClocksTypeDef RCC_Clocks;

	//skonfigurowanie miejsca przechowywania tablicy wektorow przerwan
	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);

	//konfiguracja podzialu priorytetow przerwan: 4 grupy, 4 podgrupy
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);

	//wczenie obsugi alarmu
	if(alarm_state == 1)
	{
		//konfiguracja przerwan pochodzacych od RTC Alarm
		EXTI_ClearITPendingBit(EXTI_Line17);
		EXTI_InitStructure.EXTI_Line = EXTI_Line17;
		EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
		EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
		EXTI_InitStructure.EXTI_LineCmd = ENABLE;
		EXTI_Init(&EXTI_InitStructure);
		//generowanie przerwania przez RTC Alarm
		RTC_ITConfig(RTC_IT_ALR, ENABLE);
		//odczekanie na zakoczenie operacji
		RTC_WaitForLastTask();
	}
	//wyczenie obsugi alarmu
	else
	{
		//konfiguracja przerwan pochodzacych od RTC Alarm
		EXTI_ClearITPendingBit(EXTI_Line17);
		EXTI_InitStructure.EXTI_Line = EXTI_Line17;
		EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
		EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
		EXTI_InitStructure.EXTI_LineCmd = DISABLE;
		EXTI_Init(&EXTI_InitStructure);
		//generowanie przerwania przez RTC Alarm
		RTC_ITConfig(RTC_IT_ALR, ENABLE);
		//odczekanie na zakoczenie operacji
		RTC_WaitForLastTask();
	}

	//konfiguracja przerwan pochodzacych od RTC
	NVIC_InitStructure.NVIC_IRQChannel = RTC_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	//generowanie przerwania przez RTC co 1s
	RTC_ITConfig(RTC_IT_SEC, ENABLE);
	//odczekanie na zakoczenie operacji
	RTC_WaitForLastTask();

	//konfiguracja przerwan pochodzcych od RTC Alarm
	NVIC_InitStructure.NVIC_IRQChannel = RTCAlarm_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	//ustwienie dzialania GPIO jako linii EXTI
	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource0);
	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource1);
	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource2);
	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource3);
  	GPIO_EXTILineConfig(GPIO_PortSourceGPIOC, GPIO_PinSource5);

	//ustawienie sposobu generowania przerwania
  	EXTI_InitStructure.EXTI_Line = EXTI_Line0 | EXTI_Line1 | EXTI_Line2 | EXTI_Line3 | EXTI_Line5;
  	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
  	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  	EXTI_Init(&EXTI_InitStructure);

	//konfiguracja NVIC dla przerwania EXTI0_IRQn
	NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	//konfiguracja NVIC dla przerwania EXTI1_IRQn
	NVIC_InitStructure.NVIC_IRQChannel = EXTI1_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	//konfiguracja NVIC dla przerwania EXTI2_IRQn
	NVIC_InitStructure.NVIC_IRQChannel = EXTI2_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	//konfiguracja NVIC dla przerwania EXTI3_IRQn
	NVIC_InitStructure.NVIC_IRQChannel = EXTI3_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	//konfiguracja NVIC dla przerwania EXTI9_5_IRQn
	NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	//konfiguracja przerwan pochodzacych od SysTick
	NVIC_InitStructure.NVIC_IRQChannel = SysTick_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 2;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	//uzyskanie czestotliwosci poszczegolnych linii zegara
	RCC_GetClocksFreq(&RCC_Clocks);

	//ustawienie przerwania SysTick co 100ms
	SysTick_Config(RCC_Clocks.SYSCLK_Frequency / 10000);

	freq = RCC_Clocks.SYSCLK_Frequency;
}

void Configuration_GPIO(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	//inicjalizacja pinow obslugujacych diody
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin =	GPIO_Pin_8 |
									GPIO_Pin_9 |
									GPIO_Pin_10 |
									GPIO_Pin_11 |
									GPIO_Pin_12 |
									GPIO_Pin_13 |
									GPIO_Pin_14 |
									GPIO_Pin_15;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	//wyczenie wszystkich diod LED
	LED_ALL_OFF;

	//inicjalizacja pinu piezo
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin =	GPIO_Pin_6;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	//konfiguracja pinow generujacych przerwanie JOY/Switch
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING  ;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
	GPIO_Init(GPIOC, &GPIO_InitStructure);

}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/*
 * Minimal __assert_func used by the assert() macro
 * */
void __assert_func(const char *file, int line, const char *func, const char *failedexpr)
{
  while(1)
  {}
}

/*
 * Minimal __assert() uses __assert__func()
 * */
void __assert(const char *file, int line, const char *failedexpr)
{
   __assert_func (file, line, NULL, failedexpr);
}
