/**
  ******************************************************************************
  * @file    Project/STM32F10x_StdPeriph_Template/stm32f10x_it.c 
  * @author  MCD Application Team
  * @version V3.3.0
  * @date    04/16/2010
  * @brief   Main Interrupt Service Routines.
  *          This file provides template for all exceptions handler and 
  *          peripherals interrupt service routine.
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2010 STMicroelectronics</center></h2>
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x_it.h"
#include "lib_pcd8544.h"
#include "macro.h"

/** @addtogroup STM32F10x_StdPeriph_Template
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/******************************************************************************/
/*            Cortex-M3 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
  * @brief   This function handles NMI exception.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Memory Manage exception.
  * @param  None
  * @retval None
  */
void MemManage_Handler(void)
{
  /* Go to infinite loop when Memory Manage exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Bus Fault exception.
  * @param  None
  * @retval None
  */
void BusFault_Handler(void)
{
  /* Go to infinite loop when Bus Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Usage Fault exception.
  * @param  None
  * @retval None
  */
void UsageFault_Handler(void)
{
  /* Go to infinite loop when Usage Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles Debug Monitor exception.
  * @param  None
  * @retval None
  */
void DebugMon_Handler(void)
{
}

/**
  * @brief  This function handles PendSVC exception.
  * @param  None
  * @retval None
  */
void PendSV_Handler(void)
{
}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTick_Handler(void)
{
	extern uint8_t req_alarm;

	if(req_alarm == 1)
	{
		BEEPER_INV;
	}
}

/**
  * @brief  This function handles RTC 1 sec Handler.
  * @param  None
  * @retval None
  */
void RTC_IRQHandler(void)
{
	extern uint32_t rtc_time;
	extern uint32_t rtc_alarm;
	extern uint8_t req_alarm;
	extern uint8_t set_pos;

	if(RTC_GetITStatus(RTC_IT_SEC) != RESET)
	{
		//sprawdzenie czy RTC si nie przekrci 24h
		if(RTC_GetCounter() >= (86400 - 1))
		{
			RTC_SetCounter(RTC_GetCounter() % (86400 - 1));
		}

		//pobieranie aktualnego czasu tylko gdy nie jest przeprwadzane ustawiane czasu/alarmu
		if(set_pos == 0)
		{
			rtc_time = RTC_GetCounter();
			rtc_alarm = RTC->ALRL | (RTC->ALRH << 16);
		}

		LED_1_INV;
		//wyczyszczenie bitu generujacego przerwanie
		RTC_ClearITPendingBit(RTC_IT_SEC);
		//odczekanie na zakoczenie operacji na RTC
		RTC_WaitForLastTask();
	}
}

/**
  * @brief  This function handles RTC alarm Handler.
  * @param  None
  * @retval None
  */
void RTCAlarm_IRQHandler(void)
{
	extern uint8_t req_alarm;
	if(EXTI_GetITStatus(EXTI_Line17) != RESET)
	{
		EXTI_ClearITPendingBit(EXTI_Line17);

		req_alarm = 1;
		LED_ALL_ON;
		RTC_WaitForLastTask();
		RTC_ClearITPendingBit(RTC_IT_ALR);
		RTC_WaitForLastTask();
	}
}

/**
  * @brief  This function handles External line 0 interrupt request.
  * @param  None
  * @retval None
  */
void EXTI0_IRQHandler(void)
{
	extern uint32_t rtc_time;
	extern uint32_t rtc_alarm;
	extern uint8_t set_pos;
	extern uint8_t alarm_state;

	//przerwanie wywolane przez joystick w pozycji - up
	if(EXTI_GetITStatus(EXTI_Line0) != RESET)
  	{
		switch(set_pos)
		{
			//ustawianie dziesiatki godziny czasu
			case 1 :
				if((rtc_time / 36000) < 2)
					{rtc_time = rtc_time + 36000;}
				else if((rtc_time / 36000) == 2)
					{rtc_time = rtc_time - 2*36000;}
				if(rtc_time >= 86400)
					{rtc_time = rtc_time - 3600 * (rtc_time / 3600 % 10);}
				break;
			//ustawianie jednosci godziny czasu
			case 2 :
				if((rtc_time % 36000 / 3600) < 9)
					{rtc_time = rtc_time + 3600;}
				else if((rtc_time % 36000 / 3600) == 9)
					{rtc_time = rtc_time - 9*3600;}
				if(rtc_time >= 86400)
					{rtc_time = rtc_time - 72000;}
				break;
			//ustawianie dziesiatki minuty czasu
			case 3 :
				if((rtc_time % 3600 / 600) < 5)
					{rtc_time = rtc_time + 600;}
				else if((rtc_time % 3600 / 600) == 5)
					{rtc_time = rtc_time - 5*600;}
				break;
			//ustawianie jednosci minuty czasu
			case 4 :
				if((rtc_time % 600 / 60) < 9)
					{rtc_time = rtc_time + 60;}
				else if((rtc_time % 600 / 60) == 9)
					{rtc_time = rtc_time - 9*60;}
				break;
			//ustawianie dziesiatki sekundy czasu
			case 5 :
				if((rtc_time % 60 / 10) < 5)
					{rtc_time = rtc_time + 10;}
				else if((rtc_time % 60 / 10) == 5)
					{rtc_time = rtc_time - 5*10;}
				break;
			//ustawianie jednosci sekundy czasu
			case 6 :
				if((rtc_time % 10) < 9)
					{rtc_time = rtc_time + 1;}
				else if((rtc_time % 10) == 9)
					{rtc_time = rtc_time - 9;}
				break;
			//ustawianie dziesiatki godziny alarmu
			case 7 :
				if((rtc_alarm / 36000) < 2)
					{rtc_alarm = rtc_alarm + 36000;}
				else if((rtc_alarm / 36000) == 2)
					{rtc_alarm = rtc_alarm - 2*36000;}
				if(rtc_alarm >= 86400)
					{rtc_alarm = rtc_alarm - 3600 * (rtc_alarm / 3600 % 10);}
				break;
			//ustawianie jednosci godziny alarmu
			case 8 :
				if((rtc_alarm % 36000 / 3600) < 9)
					{rtc_alarm = rtc_alarm + 3600;}
				else if((rtc_alarm % 36000 / 3600) == 9)
					{rtc_alarm = rtc_alarm - 9*3600;}
				if(rtc_alarm >= 86400)
					{rtc_alarm = rtc_alarm - 72000;}
				break;
			//ustawianie dziesiatki minuty alarmu
			case 9 :
				if((rtc_alarm % 3600 / 600) < 5)
					{rtc_alarm = rtc_alarm + 600;}
				else if((rtc_alarm % 3600 / 600) == 5)
					{rtc_alarm = rtc_alarm - 5*600;}
				break;
			//ustawianie jednosci minuty alarmu
			case 10 :
				if((rtc_alarm % 600 / 60) < 9)
					{rtc_alarm = rtc_alarm + 60;}
				else if((rtc_alarm % 600 / 60) == 9)
					{rtc_alarm = rtc_alarm - 9*60;}
				break;
			//ustawianie dziesiatki sekundy alarmu
			case 11 :
				if((rtc_alarm % 60 / 10) < 5)
					{rtc_alarm = rtc_alarm + 10;}
				else if((rtc_alarm % 60 / 10) == 5)
					{rtc_alarm = rtc_alarm - 5*10;}
				break;
			//ustawianie jednosci sekundy alarmu
			case 12 :
				if((rtc_alarm % 10) < 9)
					{rtc_alarm = rtc_alarm + 1;}
				else if((rtc_alarm % 10) == 9)
					{rtc_alarm = rtc_alarm - 9;}
				break;
			//wczenie/wyczenie alarmu
			case 13 :
					if(alarm_state == 0)
						{alarm_state = 1;}
					else
						{alarm_state = 0;}
				break;
			//
			default : break;
		}
		EXTI_ClearITPendingBit(EXTI_Line0);
  	}
}

/**
  * @brief  This function handles External line 1 interrupt request.
  * @param  None
  * @retval None
  */
void EXTI1_IRQHandler(void)
{
	extern uint32_t rtc_time;
	extern uint32_t rtc_alarm;
	extern uint8_t set_pos;
	extern uint8_t alarm_state;

	//przerwanie wywolane przez joystick w pozycji - down
	if(EXTI_GetITStatus(EXTI_Line1) != RESET)
  	{
		switch(set_pos)
		{
			//ustawianie dziesiatki godziny czasu
			case 1 :
				if((rtc_time / 36000) == 0)
					{rtc_time = rtc_time + 72000;}
				else if((rtc_time / 36000) > 0)
					{rtc_time = rtc_time - 36000;}
				if(rtc_time >= 86400)
					{rtc_time = rtc_time - 3600 * (rtc_time / 3600 % 10);}
				break;
			//ustawianie jednosci godziny czasu
			case 2 :
				if((rtc_time % 36000 / 3600) > 0)
					{rtc_time = rtc_time - 3600;}
				else if((rtc_time % 36000 / 3600) == 0)
					{rtc_time = rtc_time + 9*3600;}
				if(rtc_time >= 86400)
					{rtc_time = rtc_time - 72000;}
				break;
			//ustawianie dziesiatki minuty czasu
			case 3 :
				if((rtc_time % 3600 / 600) > 0)
					{rtc_time = rtc_time - 600;}
				else if((rtc_time % 3600 / 600) == 0)
					{rtc_time = rtc_time + 5*600;}
				break;
			//ustawianie jednosci minuty czasu
			case 4 :
				if((rtc_time % 600 / 60) > 0)
					{rtc_time = rtc_time - 60;}
				else if((rtc_time % 600 / 60) == 0)
					{rtc_time = rtc_time + 9*60;}
				break;
			//ustawianie dziesiatki sekundy czasu
			case 5 :
				if((rtc_time % 60 / 10) > 0)
					{rtc_time = rtc_time - 10;}
				else if((rtc_time % 60 / 10) == 0)
					{rtc_time = rtc_time + 5*10;}
				break;
			//ustawianie jednosci sekundy czasu
			case 6 :
				if((rtc_time % 10) > 0)
					{rtc_time = rtc_time - 1;}
				else if((rtc_time % 10) == 0)
					{rtc_time = rtc_time + 9;}
				break;
				//ustawianie dziesiatki godziny alarmu
			case 7 :
				if((rtc_alarm / 36000) == 0)
					{rtc_alarm = rtc_alarm + 72000;}
				else if((rtc_alarm / 36000) > 0)
					{rtc_alarm = rtc_alarm - 36000;}
				if(rtc_alarm >= 86400)
					{rtc_alarm = rtc_alarm - 3600 * (rtc_alarm / 3600 % 10);}
				break;
			//ustawianie jednosci godziny alarmu
			case 8 :
				if((rtc_alarm % 36000 / 3600) > 0)
					{rtc_alarm = rtc_alarm - 3600;}
				else if((rtc_alarm % 36000 / 3600) == 0)
					{rtc_alarm = rtc_alarm + 9*3600;}
				if(rtc_alarm >= 86400)
					{rtc_alarm = rtc_alarm - 72000;}
				break;
			//ustawianie dziesiatki minuty alarmu
			case 9 :
				if((rtc_alarm % 3600 / 600) > 0)
					{rtc_alarm = rtc_alarm - 600;}
				else if((rtc_alarm % 3600 / 600) == 0)
					{rtc_alarm = rtc_alarm + 5*600;}
				break;
			//ustawianie jednosci minuty alarmu
			case 10 :
				if((rtc_alarm % 600 / 60) > 0)
					{rtc_alarm = rtc_alarm - 60;}
				else if((rtc_alarm % 600 / 60) == 0)
					{rtc_alarm = rtc_alarm + 9*60;}
				break;
			//ustawianie dziesiatki sekundy alarmu
			case 11 :
				if((rtc_alarm % 60 / 10) > 0)
					{rtc_alarm = rtc_alarm - 10;}
				else if((rtc_alarm % 60 / 10) == 0)
					{rtc_alarm = rtc_alarm + 5*10;}
				break;
			//ustawianie jednosci sekundy alarmu
			case 12 :
				if((rtc_alarm % 10) > 0)
					{rtc_alarm = rtc_alarm - 1;}
				else if((rtc_alarm % 10) == 0)
					{rtc_alarm = rtc_alarm + 9;}
				break;
			//wczenie/wyczenie alarmu
			case 13 :
					if(alarm_state == 0)
						{alarm_state = 1;}
					else
						{alarm_state = 0;}
				break;
			//
			default : break;
		}
		EXTI_ClearITPendingBit(EXTI_Line1);
  	}
}

/**
  * @brief  This function handles External line 2 interrupt request.
  * @param  None
  * @retval None
  */
void EXTI2_IRQHandler(void)
{
	extern uint32_t rtc_time;
	extern uint32_t rtc_alarm;
	extern uint8_t set_pos;

	//przerwanie wywolane przez joystick w pozycji - right
	if(EXTI_GetITStatus(EXTI_Line2) != RESET)
  	{
		//zmiana pozycji
		if((set_pos > 0) && (set_pos < 13))
			{set_pos++;}
		else if(set_pos == 13)
			{set_pos = 1;}
		EXTI_ClearITPendingBit(EXTI_Line2);
  	}
}

/**
  * @brief  This function handles External line 3 interrupt request.
  * @param  None
  * @retval None
  */
void EXTI3_IRQHandler(void)
{
	extern uint32_t rtc_time;
	extern uint32_t rtc_alarm;
	extern uint8_t set_pos;

	//przerwanie wywolane przez joystick w pozycji - left
	if(EXTI_GetITStatus(EXTI_Line3) != RESET)
  	{
		//zmiana pozycji
		if((set_pos > 1) && (set_pos < 14))
			{set_pos--;}
		else if(set_pos == 1)
			{set_pos = 13;}
		EXTI_ClearITPendingBit(EXTI_Line3);
  	}
}

/**
  * @brief  This function handles External line 5 interrupt request.
  * @param  None
  * @retval None
  */

void EXTI9_5_IRQHandler(void)
{
	EXTI_InitTypeDef EXTI_InitStructure;
	extern uint32_t rtc_time;
	extern uint32_t rtc_alarm;
	extern uint8_t set_pos;
	extern uint8_t req_alarm;
	extern uint8_t alarm_state;

	//przerwanie wywolane przez joystick w pozycji - enter
	if(EXTI_GetITStatus(EXTI_Line5) != RESET)
  	{
		if((set_pos == 0) && (req_alarm == 1))
		{
			req_alarm = 0;
			LED_ALL_OFF;
		}
		else if(set_pos == 0)
		{
			set_pos = 1;
			//wyczenie obsugi alarmu podczas edycji czasu i alarmu
			EXTI_ClearITPendingBit(EXTI_Line17);
			EXTI_InitStructure.EXTI_Line = EXTI_Line17;
			EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
			EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
			EXTI_InitStructure.EXTI_LineCmd = DISABLE;
			EXTI_Init(&EXTI_InitStructure);
			RTC_ITConfig(RTC_IT_ALR, DISABLE);
			//dioda informacyjna o edycji
			LED_2_ON;
		}
		else
		{

			//zresetowanie flagi SECF rejestru RTC->CRL, wymagane do poprawnego zapisu rejestru alarmu
			RTC_ClearFlag(RTC_FLAG_SEC);
			//while(RTC_GetFlagStatus(RTC_FLAG_SEC) == RESET){RTC_ClearFlag(RTC_FLAG_SEC);};
			//odczekanie na zakoczenie operacji na RTC
			RTC_WaitForLastTask();

			//ustawienie rejestru alarmu RTC na 12h00m00s
			RTC_SetAlarm(rtc_alarm);
			//odczekanie na zakoczenie operacji na RTC
			RTC_WaitForLastTask();

			//ustawienie czasu
			RTC_SetCounter(rtc_time);
			RTC_WaitForLastTask();

			//wyjscie z edycji
			set_pos = 0;

			//wczenie/wyczenie obsugi alarmu
			if(alarm_state == 1)
			{
				EXTI_ClearITPendingBit(EXTI_Line17);
				EXTI_InitStructure.EXTI_Line = EXTI_Line17;
				EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
				EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
				EXTI_InitStructure.EXTI_LineCmd = ENABLE;
				EXTI_Init(&EXTI_InitStructure);
				RTC_ITConfig(RTC_IT_ALR, ENABLE);
			}
			else
			{
				EXTI_ClearITPendingBit(EXTI_Line17);
				EXTI_InitStructure.EXTI_Line = EXTI_Line17;
				EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
				EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
				EXTI_InitStructure.EXTI_LineCmd = DISABLE;
				EXTI_Init(&EXTI_InitStructure);
				RTC_ITConfig(RTC_IT_ALR, DISABLE);
			}
			//dioda informacyjna o edycji
			LED_2_OFF;
		}
		EXTI_ClearITPendingBit(EXTI_Line5);
  	}
}

/******************************************************************************/
/*                 STM32F10x Peripherals Interrupt Handlers                   */
/*  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the  */
/*  available peripheral interrupt handler's name please refer to the startup */
/*  file (startup_stm32f10x_xx.s).                                            */
/******************************************************************************/

/**
  * @brief  This function handles PPP interrupt request.
  * @param  None
  * @retval None
  */
/*void PPP_IRQHandler(void)
{
}*/

/**
  * @}
  */ 


/******************* (C) COPYRIGHT 2010 STMicroelectronics *****END OF FILE****/
